import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.Files;
import java.util.Properties;

/*
 * Class provides read access to the MongoDB connection
 * information (host and port) stored in a properties file.
 */
public class AppProperties {

	// Represents persistent set of properties, 
	// (key/value pairs that can retrieved).
	private Properties properties;

	// Property keys
	public static final String HOST_K = "host";
	public static final String PORT_K = "port";

	// Location where the file is stored.
	private static final String BASE_DIRECTORY =
			System.getProperty("user.dir");
	// The properties file name.		
	private static final String APP_PROPS_FILE =
			"application_properties.properties";


	/*
	 * Constructs an instance.
	 * Loads the properties from the file.
	 * Throws an runtime exception in case of a problem with the properties file.
	 */
	public AppProperties() {

		try {
			properties = loadPropertiesFromFile();
		}
		catch(IOException e) {
			System.out.println("Error reading the properties file: " + APP_PROPS_FILE);
			throw new RuntimeException(e.toString());
		}
	}

	/*
	 * Returns the value of the specified property.
	 */
	public String getPropertyValue(String propertyName) {

		return properties.getProperty(propertyName);
	}

	/*
	 * Loads and returns the properties from the file.
	 * Throws IOException if there is a problem with the file.
	 */
	private Properties loadPropertiesFromFile()
			throws IOException {

		Path path = Paths.get(BASE_DIRECTORY, APP_PROPS_FILE);
		InputStream fis = Files.newInputStream(path);
		Properties loadedProperties = new Properties();
		loadedProperties.load(fis);
		fis.close();
		return loadedProperties;
	}
}