import java.util.List;
import java.util.ArrayList;
import org.bson.Document;
import org.bson.conversions.Bson;
import com.mongodb.client.MongoClient;
import com.mongodb.client.MongoClients;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.result.*;
import com.mongodb.client.model.Updates;
import com.mongodb.client.model.Filters;
import com.mongodb.MongoException;


/**
 * Class has MongoDB database connection creation and CRUD operation methods.
 * See Main.java, where this class is accessed.
 */
public class MongoOps {

	private MongoClient mongoClient;
	private MongoCollection<Document> collection;


	/*
	 * Constructor with the input application properties to build the connection URL.
	 * Gets connection to the database and creates an instance of the collection.
	 * Throws a RuntimeException if the conection URL, the database namespace or
	 * the collection name are not valid.
	 */
	public MongoOps(AppProperties props) {
	
		try {
			final String URL = "mongodb://" + 
								props.getPropertyValue(AppProperties.HOST_K) + 
								":" + 
								props.getPropertyValue(AppProperties.PORT_K) + 
								"/";
			mongoClient = MongoClients.create(URL);
			collection = mongoClient
								.getDatabase("crud")
								.getCollection("test");
		}
		catch(RuntimeException e) {
			System.out.println("Database connection format, database or collection name error.");
			throw new RuntimeException(e);
		}
		
		System.out.println("Connected to MongoDB database.");
	}
	
	/*
	 * Routine to close the database connection.
	 */
	public void close() {

		if (mongoClient != null) {
		
			mongoClient.close();
		}
		
		System.out.println("\nMongoDB database connection closed.");
	}

	/*
	 * Returns the document inserted in the database collection using
	 * the input field values. The inserted document, in addition,
	 * has the _id field, which was generated by the driver software
	 * as part of the insert operation.
	 * Throws a RuntimeException if there is a database error during the write.
	 */	
	public Document insertDoc(String name, String city) {

		Document doc = new Document("name", name).append("city", city);

		try {
			collection.insertOne(doc);
		}
		catch(MongoException e) {
			System.out.println("Database insert error.");
			throw new RuntimeException(e.getMessage());
		}
		
		return doc;
	}

	/*
	 * Returns the number of documents updated in the database collection
	 * using the input field values. Updates the first matching document.
	 * Throws a RuntimeException if there is a database error during the write.
	 */		
	public long updateDoc(String name, String city) {
	
		Bson filter = Filters.eq("name", name);
		Bson update = Updates.set("city", city);
		UpdateResult result = null;
		
		try {
			result = collection.updateOne(filter, update);
		}
		catch(MongoException e) {
			System.out.println("Database update error.");
			throw new RuntimeException(e.getMessage());
		}
		
		return result.getModifiedCount();
	}

	/*
	 * Returns the number of  documents deleted from the database collection
	 * using the input Name field value. Deletes the first matching document.
	 * Throws a RuntimeException if there is a database error during the write.
	 */	
	public long deleteDoc(String name) {
	
		Bson filter = new Document().append("name", name);
		DeleteResult result = null;
		
		try {
			result = collection.deleteOne(filter);		
		}
		catch(MongoException e) {
			System.out.println("Database delete error.");
			throw new RuntimeException(e.getMessage());
		}

		return result.getDeletedCount();
	}
	
	/*
	 * Returns all documents from the database collection.
	 */
	public List<Document> getAll() {

		return collection.find().into(new ArrayList<Document>());	
	}

	/*
	 * Returns the documents from the database collection based upon
	 * the input field values.
	 * - When both values are entered, the database is queried with
	 * Name -and- City.
	 * - When only one field value is entered, the database is queried
	 * with -that- field only.
	 */	
	public List<Document> queryByField(String name, String city) {

		Bson filter = null;

		if (name.isEmpty()) {
			// Query by City
			filter = Filters.eq("city", city);
		}
		else
		if (city.isEmpty()) {
			// Query by Name			
			filter = Filters.eq("name", name);
		}
		else {
			// Query with Name and City
			filter = Filters.and(Filters.eq("name", name), Filters.eq("city", city));
		}
		
		return collection.find(filter).into(new ArrayList<Document>());
	}

}